@@ -1,21 +1,27 @@
+# Copyright Contributors to the Amundsen project.
+# SPDX-License-Identifier: Apache-2.0
+
 import ast
 import importlib
-import os
 import logging
 import logging.config
+import os
 import sys
+from typing import Any, Dict  # noqa: F401
 
-from flask import Flask, Blueprint
-from flask_restful import Api
-from flask_cors import CORS
-from typing import Dict, Any  # noqa: F401
 from flasgger import Swagger
+from flask import Blueprint, Flask
+from flask_cors import CORS
+from flask_restful import Api
 
 from search_service.api.dashboard import SearchDashboardAPI, SearchDashboardFilterAPI
+from search_service.api.document import (
+    DocumentFeatureAPI, DocumentFeaturesAPI, DocumentTableAPI, DocumentTablesAPI, DocumentUserAPI, DocumentUsersAPI,
+)
+from search_service.api.feature import SearchFeatureAPI, SearchFeatureFilterAPI
+from search_service.api.healthcheck import healthcheck
 from search_service.api.table import SearchTableAPI, SearchTableFilterAPI
 from search_service.api.user import SearchUserAPI
-from search_service.api.document import DocumentUserAPI, DocumentTableAPI, DocumentTablesAPI, DocumentUsersAPI
-from search_service.api.healthcheck import healthcheck
 
 # For customized flask use below arguments to override.
 FLASK_APP_MODULE_NAME = os.getenv('FLASK_APP_MODULE_NAME')
@@ -93,14 +99,21 @@ def create_app(*, config_module_class: str) -> Flask:
     api.add_resource(SearchDashboardAPI, '/search_dashboard')
     api.add_resource(SearchDashboardFilterAPI, '/search_dashboard_filter')
 
+    # Feature Search API
+    api.add_resource(SearchFeatureAPI, '/search_feature')
+    api.add_resource(SearchFeatureFilterAPI, '/search_feature_filter')
+
     # DocumentAPI
-    # todo: needs to update to handle dashboard/user or other entities use cases.
+    # todo: needs to handle dashboard
     api.add_resource(DocumentTablesAPI, '/document_table')
     api.add_resource(DocumentTableAPI, '/document_table/<document_id>')
 
     api.add_resource(DocumentUsersAPI, '/document_user')
     api.add_resource(DocumentUserAPI, '/document_user/<document_id>')
 
+    api.add_resource(DocumentFeaturesAPI, '/document_feature')
+    api.add_resource(DocumentFeatureAPI, '/document_feature/<document_id>')
+
     app.register_blueprint(api_bp)
 
     if app.config.get('SWAGGER_ENABLED'):
