@@ -1,6 +1,9 @@
+# Copyright Contributors to the Amundsen project.
+# SPDX-License-Identifier: Apache-2.0
+
 import logging
 from http import HTTPStatus
-from typing import Iterable, Any
+from typing import Any, Iterable
 
 from flasgger import swag_from
 from flask_restful import Resource, reqparse  # noqa: I201
@@ -10,7 +13,6 @@ from search_service.exception import NotFoundException
 from search_service.models.dashboard import SearchDashboardResultSchema
 from search_service.proxy import get_proxy_client
 
-
 DASHBOARD_INDEX = 'dashboard_search_index'
 
 LOGGING = logging.getLogger(__name__)
@@ -48,7 +50,7 @@ class SearchDashboardAPI(Resource):
                 index=args['index']
             )
 
-            return SearchDashboardResultSchema().dump(results).data, HTTPStatus.OK
+            return SearchDashboardResultSchema().dump(results), HTTPStatus.OK
 
         except NotFoundException:
             return {'message': 'query_term does not exist'}, HTTPStatus.NOT_FOUND
@@ -64,6 +66,7 @@ class SearchDashboardFilterAPI(BaseFilterAPI):
     """
     Search Filter for Dashboard
     """
+
     def __init__(self) -> None:
         super().__init__(schema=SearchDashboardResultSchema,
                          index=DASHBOARD_INDEX)
